#function that reads data from a csv file and formats it in a way that the rest of the
#program can interpret
def read(fileName: str):

#reads data from a file
    data = open(fileName, "r")

    data = data.read()
    data = data.split("\n")

    while (data[-1] == ""):
        data.pop()

    dataType = [] #variable that saves datatype of first row in each column

    inconsistentCell = None #last cell that has inconsistent data in the column (if exists)

#loops through each row of the csv (i is the row index)
    for i in range(0,len(data)):
        data[i] = data[i].split(",")

#loops through each column at row i (j is the column index)
        for j in range(0,len(data[i])):

            data[i][j] = data[i][j].strip("\"")

#if the data can be converted to int it will be, otherwise it'll be converted to
#float, and if it can't convert to float it will stay as a string
            try:
                data[i][j] = int(data[i][j])
            except:
                try:
                    data[i][j] = float(data[i][j])
                except:
                    pass

            if i == 1:
                dataType.append(type(data[i][j]))
            elif i > 1 and type(data[i][j]) != dataType[j]:
                inconsistentCell = (i+1,j+1)
                
    return (data, inconsistentCell)

def saveScatter(*, xs: list, ys: list, groups: list = None, title: str = None,
                  xLabel: str = None, yLabel: str = None, gridLines: str = "", fileName: str):
    
    fw = open(fileName, "w")
    fw.write("scatter\n")
    for x in xs:
        x = str(x).replace(" ", "**SPACE**")
        fw.write(x+" ")
    fw.write("\n")
    for y in ys:
        y = str(y).replace(" ", "**SPACE**")
        fw.write(y+" ")
    fw.write("\n")

    if groups != None:
        for group in groups[0]:
            group = str(group).replace(" ","**SPACE**")
            fw.write(group+" ")
    else:
        fw.write(" ")

    fw.write("\n")

    if title != None or title != "":
        fw.write(title+"\n")
    else:
        fw.write(" \n")

    if xLabel != None or xLabel != "":
        fw.write(xLabel+"\n")
    else:
        fw.write(" \n")

    if yLabel != None or yLabel !=  "":
        fw.write(yLabel+"\n")
    else:
        fw.write(" \n")

    fw.write(gridLines)
    fw.close()

def saveInterval(*,data: dict, title: str = None, xLabel: str = None,
                  yLabel: str = None, gridLines: str = "", groupNames: tuple=(), fileName: str):
    
    fw = open(fileName, "w")
    fw.write("interval\n")

    dataKeys = []
    dataUppers = []
    dataMeans = []
    dataLowers = []

    for key,value in data.items():

        key = key.replace("\n", "\\n")
        key = key.replace(" ", "**SPACE**")

        dataKeys.append(key)
        dataUppers.append(value[0])
        dataMeans.append(value[1])
        dataLowers.append(value[2])

    for key in dataKeys:
        fw.write(str(key)+" ")
    fw.write("\n")

    for upper in dataUppers:
        fw.write(str(upper)+" ")
    fw.write("\n")

    for mean in dataMeans:
        fw.write(str(mean)+" ")
    fw.write("\n")

    for lower in dataLowers:
        fw.write(str(lower)+" ")
    fw.write("\n")

    if title != None or title != "":
        fw.write(title+"\n")
    else:
        fw.write(" \n")

    if xLabel != None or xLabel != "":
        fw.write(xLabel+"\n")
    else:
        fw.write(" \n")

    if yLabel != None or yLabel !=  "":
        fw.write(yLabel+"\n")
    else:
        fw.write(" \n")

    for name in groupNames:
        name = name.replace(" ", "**SPACE**")
        fw.write(name+" ")
    fw.write("\n")

    fw.write(gridLines)
    fw.close()

def saveHist(*, xs: list, bins: int = None, title: str = None, xLabel: str = None,
                  yLabel: str = None, gridLines: str = "", fileName: str):

    fw = open(fileName, "w")

    fw.write("histogram\n")

    for x in xs:
        x = str(x).replace(" ", "**SPACE**")
        fw.write(str(x)+" ")
    fw.write("\n")

    if bins != None:
        fw.write(str(bins) + "\n")
    else:
        fw.write(" \n")

    if title != None or title != "":
        fw.write(title+"\n")
    else:
        fw.write(" \n")

    if xLabel != None or xLabel != "":
        fw.write(xLabel+"\n")
    else:
        fw.write(" \n")

    if yLabel != None or yLabel !=  "":
        fw.write(yLabel+"\n")
    else:
        fw.write(" \n")

    fw.write(gridLines)
    fw.close()
    

def loadFigure(fileName):
    
    fr = open(fileName, "r")

    lines = fr.read().splitlines()

    figType = lines[0]

    if figType == "scatter":

        xs = lines[1].split()
        ys = lines[2].split()

        groups = []
        groups.append([])
        groups[0] = lines[3].split()

        data = (xs, ys)

        for i in range(len(data)):
            for j in range(len(data[i])):

                data[i][j] = data[i][j].replace("**SPACE**", " ")

                try:
                    data[i][j] = int(data[i][j])
                except:
                    try:
                        data[i][j] = float(data[i][j])
                    except:
                        pass

        for i in range(len(groups[0])):
            groups[0][i] = groups[0][i].replace("**SPACE**", " ")
            try:
                groups[0][i] = int(groups[0][i])
            except:
                try:
                    groups[0][i] = float(groups[0][i])
                except:
                    pass

        if len(groups[0]) < 2:
            groups = None

        title = lines[4]
        xLabel = lines[5]
        yLabel = lines[6]

        if(len(lines) > 7):
            gridLines = lines[7]
        else:
            gridLines = ""

        return (xs, ys, groups, title, xLabel, yLabel, gridLines)

    elif figType == "interval":
        
        keys = lines[1].split()
        uppers = lines[2].split()
        means = lines[3].split()
        lowers = lines[4].split()

        for i in range(len(keys)):
            keys[i] = keys[i].replace("\\n", "\n")
            keys[i] = keys[i].replace("**SPACE**", " ")

        for i in range(len(uppers)):
            if uppers[i] == "None":
                uppers[i] = None
            if lowers[i] == "None":
                lowers[i] = None

        data = (keys, uppers, means, lowers)

        for i in range(len(data)):
            for j in range(len(data[i])):
                try:
                    data[i][j] = int(data[i][j])
                except:
                    try:
                        data[i][j] = float(data[i][j])
                    except:
                        pass

        dataDict = {}

        for i in range(len(keys)):
            dataDict[keys[i]] = (uppers[i], means[i], lowers[i])

        title = lines[5]
        xLabel = lines[6]
        yLabel = lines[7]

        groupNames = []

        groupNamesLoad = lines[8].split()

        for name in groupNamesLoad:
            name = name.replace("**SPACE**", " ")
            groupNames.append(name)
        groupNames = tuple(groupNames)

        if(len(lines) > 9):
            gridLines = lines[9]
        else:
            gridLines = ""

        return (dataDict, title, xLabel, yLabel, gridLines, groupNames)

    elif figType == "histogram":
        
        xs = lines[1].split()

        for i in range(len(xs)):

            xs[i] = xs[i].replace("**SPACE**", " ")

            try:
                xs[i] = int(xs[i])
            except:
                try:
                    xs[i] = float(xs[i])
                except:
                    pass

        if lines[2] == " ":
            bins = None
        else:
            bins = int(lines[2])

        title = lines[3]
        xLabel = lines[4]
        yLabel = lines[5]

        if(len(lines) > 6):
            gridLines = lines[6]
        else:
            gridLines = ""

        return (xs, bins, title, xLabel, yLabel, gridLines)
        
        
        


def getFigType(fileName):
    fr = open(fileName, "r")
    lines = fr.read().splitlines()
    return lines[0]


import tkinter as tk
from tkinter import messagebox
from tkinter import ttk
from tkinter import filedialog
from tkinter import font

from . import rw
from . import calculation
from . import graphing

class IntervalFrame:

    def __init__(self, frame):

        self.frame = frame

        self.iTypeSelection = tk.StringVar()

        self.yVarSelection = tk.StringVar()

        self.xGridCheckVal = tk.StringVar()
        self.yGridCheckVal = tk.StringVar()
        

#Column 0
#====================================================================

        frame.grid_columnconfigure(0, pad=25)

        self.cVarLabel = tk.Label(frame, text="X-Axis Columns")
        self.cVarLabel.grid(row=3, column=0)

        self.cVar = tk.Listbox(frame, height=4, width=23)
        self.cVar.grid(row=4, column=0, rowspan=3)

        self.yVarLabel = tk.Label(frame, text="Y-Axis Column *")
        self.yVarLabel.grid(row=7, column=0, columnspan=3, pady=(5,0))

        self.yVar = ttk.Combobox(frame, textvariable=self.yVarSelection, values=[], state="readonly")
        self.yVar.grid(row=8, column=0, columnspan=3)

        self.xGridCheck = tk.Checkbutton(frame, variable=self.xGridCheckVal, onvalue="x", offvalue="", text="X-axis grid lines")
        self.xGridCheck.grid(row=9, column=0, columnspan=3)

        self.yGridCheck = tk.Checkbutton(frame, variable=self.yGridCheckVal, onvalue="y", offvalue="", text="Y-axis grid lines")
        self.yGridCheck.grid(row=10, column=0, columnspan=3)


#Column 1
#====================================================================

        self.deselectButton = tk.Button(frame, text="⤺")
        self.deselectButton["font"] = font.Font(size=12)
        self.deselectButton.bind("<Button-1>", self.removeCategory)
        self.deselectButton.grid(row=4, column=1)

        self.selectButton = tk.Button(frame, text="⤻")
        self.selectButton["font"] = font.Font(size=12)
        self.selectButton.bind("<Button-1>", self.addCategory)
        self.selectButton.grid(row=6, column=1)

#Column 2
#====================================================================

        frame.grid_columnconfigure(2, pad=25)

        self.iTypeLabel = tk.Label(frame, text="Interval Type *")
        self.iTypeLabel.grid(row=0, column=2)

        self.iType = ttk.Combobox(frame, textvariable=self.iTypeSelection, values=["Standard Error","Standard Deviation","Confidence Interval"], state="readonly")
        self.iType.grid(row=1, column=2, pady=(0,5))

        self.cVarSelectedLabel = tk.Label(frame, text="Selected X-Axis Columns\n(Outermost first)")
        self.cVarSelectedLabel.grid(row=2, column=2, rowspan=2)

        self.cVarSelected = tk.Listbox(frame, height=4, width=23)
        self.cVarSelected.grid(row=4, column=2, rowspan=3)

#Column 3
#====================================================================
  
        frame.grid_columnconfigure(3, pad=25)

        self.titleText = tk.Label(frame, text="Graph Title")
        self.titleText.grid(row=3, column=3)

        self.title = tk.Entry(frame, width=23)
        self.title.grid(row=4, column=3)

        self.xLabelText = tk.Label(frame, text="X-Axis Label")
        self.xLabelText.grid(row=5, column=3)

        self.xLabel = tk.Entry(frame, width=23)
        self.xLabel.grid(row=6, column=3)

        self.yLabelText = tk.Label(frame, text="Y-Axis Label")
        self.yLabelText.grid(row=7, column=3)

        self.yLabel = tk.Entry(frame, width=23)
        self.yLabel.grid(row=8, column=3)

        self.graphButton = tk.Button(frame, text="Create Interval Plot", command=self.createInterval)
        self.graphButton.grid(row=9, column=3, pady=(10,1))

        self.saveButton = tk.Button(frame, text=" Save Interval Plot ", command=self.saveInterval)
        self.saveButton.grid(row=10, column=3)



#====================================================================
#============================Functions===============================
#====================================================================


#code that sets the up the data to view for the interval frame
    def setFrame(self, columnLabels: list, data: list):

        self.columnLabels = columnLabels
        self.data = data

        yLabels = []

        for i in range(len(columnLabels)):
            try:
                float(data[0][i])
                yLabels.append(columnLabels[i])
            except:
                pass

        
        self.yVar.config(values=yLabels)

        self.cVar.delete(0, tk.END)
        self.cVarSelected.delete(0, tk.END)

        for label in columnLabels:
            self.cVar.insert("end", label)

        self.yVarSelection.set("Select a Column")
        self.iTypeSelection.set("Select Type")

        self.xLabel.delete(0, tk.END)
        self.yLabel.delete(0, tk.END)
        self.title.delete(0, tk.END)

        self.xGridCheckVal.set("")
        self.yGridCheckVal.set("")

    def addCategory(self, event):

        if self.cVar.curselection():
            selectedIndex = self.cVar.curselection()[0]
            selectedValue = self.cVar.get(selectedIndex)

            self.cVarSelected.insert(tk.END, selectedValue)
            self.cVar.delete(selectedIndex)

    def removeCategory(self, event):

        if self.cVarSelected.curselection():
            selectedIndex = self.cVarSelected.curselection()[0]
            selectedValue = self.cVarSelected.get(selectedIndex)

            self.cVar.insert(self.columnLabels.index(selectedValue), selectedValue)
            self.cVarSelected.delete(selectedIndex)

    def createInterval(self):

        if(self.iType.current() == (-1)):
            messagebox.showinfo("Error", "Please select an interval type")
            return
        elif(self.yVar.current() == (-1)):
            messagebox.showinfo("Error", "Please select a column for the Y-axis")
            return

        if(self.yLabel.get() == ""):
            yLabel = self.yVarSelection.get()
        else:
            yLabel = self.yLabel.get()
        if(self.title.get() == ""):
            title = self.iTypeSelection.get() + " of " + self.yVarSelection.get()
        else:
            title = self.title.get()
        if(self.xLabel.get() == "" and len(self.cVarSelected.get(0, tk.END)) == 1):
            xLabel = self.cVarSelected.get(0)
        else:
            xLabel = self.xLabel.get()

        typeParam = ""

        if(self.iType.current() == 0):
            typeParam = "se"
        elif(self.iType.current() == 1):
            typeParam = "sd"
        elif(self.iType.current() == 2):
            typeParam = "ci"


        cVals = self.cVarSelected.get(0, tk.END)
        cIndices = []

        if len(cVals) > 0:
            for val in cVals:
                cIndices.append(self.columnLabels.index(val))
        else:
            cIndices = None

        rawGraphData = calculation.getColumns(data=self.data,
                                           yCol=self.columnLabels.index(self.yVarSelection.get()), groups=cIndices)

        graphData = calculation.getIntervals(type=typeParam, ys=rawGraphData[1], groups=rawGraphData[2])

        graphing.graphInterval(data=graphData, title=title, xLabel=xLabel,
                                yLabel=yLabel, gridLines=self.xGridCheckVal.get()+self.yGridCheckVal.get(), groupNames=cVals, show=True)

    def saveInterval(self):
        
        if(self.iType.current() == (-1)):
            messagebox.showinfo("Error", "Please select an interval type")
            return
        elif(self.yVar.current() == (-1)):
            messagebox.showinfo("Error", "Please select a column for the Y-axis")
            return

        if(self.yLabel.get() == ""):
            yLabel = self.yVarSelection.get()
        else:
            yLabel = self.yLabel.get()
        if(self.title.get() == ""):
            title = self.iTypeSelection.get() + " of " + self.yVarSelection.get()
        else:
            title = self.title.get()
        if(self.xLabel.get() == "" and len(self.cVarSelected.get(0, tk.END)) == 1):
            xLabel = self.cVarSelected.get(0)
        else:
            xLabel = self.xLabel.get()

        typeParam = ""

        if(self.iType.current() == 0):
            typeParam = "se"
        elif(self.iType.current() == 1):
            typeParam = "sd"
        elif(self.iType.current() == 2):
            typeParam = "ci"


        cVals = self.cVarSelected.get(0, tk.END)
        cIndices = []

        if len(cVals) > 0:
            for val in cVals:
                cIndices.append(self.columnLabels.index(val))
        else:
            cIndices = None

        rawGraphData = calculation.getColumns(data=self.data,
                                           yCol=self.columnLabels.index(self.yVarSelection.get()), groups=cIndices)

        graphData = calculation.getIntervals(type=typeParam, ys=rawGraphData[1], groups=rawGraphData[2])

        fileName = filedialog.asksaveasfilename(title = "Save File", defaultextension=".plot")

        if fileName != "":
            rw.saveInterval(data=graphData, title=title, xLabel=xLabel,
                                    yLabel=yLabel, gridLines=self.xGridCheckVal.get()+self.yGridCheckVal.get(), groupNames=cVals, fileName=fileName)


import tkinter as tk
from tkinter import messagebox
from tkinter import ttk
from tkinter import filedialog

from . import rw
from . import calculation
from . import graphing

class HistFrame:

    def __init__(self, frame):

        self.frame = frame

        self.xVarSelection = tk.StringVar()
        self.binTypeSelection = tk.StringVar()
        self.binNumSelection = tk.StringVar()
        
        self.xGridCheckVal = tk.StringVar()
        self.yGridCheckVal = tk.StringVar()

        #Column 0
#====================================================================

        frame.grid_columnconfigure(0, pad=25)

        self.xVarLabel = tk.Label(frame, text="X-Axis Column *")
        self.xVarLabel.grid(row=0, column=0)

        self.xVar = ttk.Combobox(frame, textvariable=self.xVarSelection, values=[], state="readonly")
        self.xVar.grid(row=1, column=0)

        self.binTypeLabel = tk.Label(frame, text="Bin Type")
        self.binTypeLabel.grid(row=2, column=0)

        self.binType = ttk.Combobox(frame, textvariable=self.binTypeSelection, values=["Automatic","Manual"], state="readonly")
        self.binType.bind("<<ComboboxSelected>>", self.binTypeSelected)
        self.binType.grid(row=3, column=0)

        self.binNumLabel = tk.Label(frame, text="Number of bins")
        self.binNumLabel.grid(row=4, column=0)

        vcmd = (frame.register(self.validate))
        self.binNum = tk.Entry(frame, textvariable=self.binNumSelection, width=23, validate="all", validatecommand=(vcmd, "%P"))
        self.binNum.grid(row=5, column=0)

        self.xGridCheck = tk.Checkbutton(frame, variable=self.xGridCheckVal, onvalue="x", offvalue="", text="X-axis grid lines")
        self.xGridCheck.grid(row=6, column=0)

        self.yGridCheck = tk.Checkbutton(frame, variable=self.yGridCheckVal, onvalue="y", offvalue="", text="Y-axis grid lines")
        self.yGridCheck.grid(row=7, column=0)

        

#Column 1
#====================================================================

        
        frame.grid_columnconfigure(1, pad=25)

        self.titleText = tk.Label(frame, text="Graph Title")
        self.titleText.grid(row=0, column=1)

        self.title = tk.Entry(frame, width=23)
        self.title.grid(row=1, column=1)

        self.xLabelText = tk.Label(frame, text="X-Axis Label")
        self.xLabelText.grid(row=2, column=1)

        self.xLabel = tk.Entry(frame, width=23)
        self.xLabel.grid(row=3, column=1)

        self.yLabelText = tk.Label(frame, text="Y-Axis Label")
        self.yLabelText.grid(row=4, column=1)

        self.yLabel = tk.Entry(frame, width=23)
        self.yLabel.grid(row=5, column=1)

        self.graphButton = tk.Button(frame, text="Create Histogram", command=self.createHist)
        self.graphButton.grid(row=6, column=1, pady=(10,1))

        self.saveButton = tk.Button(frame, text=" Save Histogram ", command=self.saveHist)
        self.saveButton.grid(row=7, column=1)

#====================================================================
#============================Functions===============================
#====================================================================

    def setFrame(self, columnLabels: list, data: list):
        
        self.columnLabels = columnLabels
        self.data = data

        self.xVar.config(values=columnLabels)

        self.xVarSelection.set("Select a Column")
        self.binType.set("Automatic")

        self.xLabel.delete(0, tk.END)
        self.yLabel.delete(0, tk.END)
        self.title.delete(0, tk.END)

        self.binNumLabel["state"] = "disabled"
        self.binNum["state"] = "disabled"

        self.xGridCheckVal.set("")
        self.yGridCheckVal.set("")

#validation function for the bin number entry
    def validate(self, P):
            if(str.isdigit(P)):
                if(len(self.binNum.get()) == 0 and int(P) > 0):
                    return True
                elif(len(self.binNum.get()) != 0):
                    return True
                else:
                    return False
            elif(P == ""):
                return True
            else:
                return False

#event handler when the bintype changes, disables/enables bin number
    def binTypeSelected(self, event):

        if self.binTypeSelection.get() == "Automatic":
            self.binNumLabel["state"] = "disabled"
            self.binNum["state"] = "disabled"
            self.binNumSelection.set("")
        elif self.binTypeSelection.get() == "Manual":
            self.binNumLabel["state"] = "normal"
            self.binNum["state"] = "normal"

#event handler that uses the gui specified data to create a histogram
    def createHist(self):
        
        if(self.xVar.current() == (-1)):
            messagebox.showinfo("Error", "Please select a column for the X-axis")
            return

        if(self.xLabel.get() == ""):
            xLabel = self.columnLabels[self.xVar.current()]
        else:
            xLabel = self.xLabel.get()
        if(self.yLabel.get() == ""):
            yLabel = "Count"
        else:
            yLabel = self.yLabel.get()
        if(self.title.get() == ""):
            title = ("Histogram of " + self.columnLabels[self.xVar.current()])
        else:
            title = self.title.get()

        graphData = calculation.getColumns(data=self.data, xCol=self.xVar.current())

        if(self.binNumSelection.get() != ""):
            bins = int(self.binNumSelection.get())
        else:
            bins = None

        graphing.graphHist(xs=graphData[0], bins=bins, xLabel=xLabel,
                              yLabel=yLabel, title=title, gridLines=self.xGridCheckVal.get()+self.yGridCheckVal.get(), show = True)
        

#event handler that prompts the user to save the histogram to a file
    def saveHist(self):
        
        if(self.xVar.current() == (-1)):
            messagebox.showinfo("Error", "Please select a column for the X-axis")
            return

        if(self.xLabel.get() == ""):
            xLabel = self.columnLabels[self.xVar.current()]
        else:
            xLabel = self.xLabel.get()
        if(self.yLabel.get() == ""):
            yLabel = "Count"
        else:
            yLabel = self.yLabel.get()
        if(self.title.get() == ""):
            title = ("Histogram of " + self.columnLabels[self.xVar.current()])
        else:
            title = self.title.get()

        graphData = calculation.getColumns(data=self.data, xCol=self.xVar.current())

        if(self.binNumSelection.get() != ""):
            bins = int(self.binNumSelection.get())
        else:
            bins = None

        fileName = filedialog.asksaveasfilename(title = "Save File", defaultextension=".plot")

        if fileName != "":
            rw.saveHist(xs=graphData[0], bins=bins, xLabel=xLabel,
                                yLabel=yLabel, title=title, gridLines=self.xGridCheckVal.get()+self.yGridCheckVal.get(), fileName=fileName)